### Figure 2.5
## ARIMA
source('bookfunctions.r')
Month ="2015-04-"
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
wsdata = normalize(ws)  
wsdata = wsdata[grep(Month,wsdata$Date),]
p = c(1:6)
q = 0
d = 0
n = nrow(wsdata)
bic = NULL
for (i in p){
  fit<-arima(wsdata$NormalizedWindspeed, order = c(p[i],d,q))
  bic[i] = BIC(fit)
}
bic
library(forecast)
NormWindspeed_roll = wsdata$NormalizedWindspeed[1:(nrow(wsdata)-6)]
pointestroll = NULL
CI_UB = NULL
CI_LB = NULL
for (i in 1:6){
  fit = arima(NormWindspeed_roll,order = c(1,0,0))
  forecastroll = forecast(NormWindspeed_roll,1,model = fit)
  pointestroll[i] = forecastroll$mean
  CI_UB[i] = forecastroll$upper[2]
  CI_LB[i] = forecastroll$lower[2]
  NormWindspeed_roll = c(NormWindspeed_roll,wsdata$NormalizedWindspeed[(length(NormWindspeed_roll)+1)])
}
pointestroll = unnormalize(data = pointestroll, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
CI_UB = unnormalize(data = CI_UB, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
CI_LB = unnormalize(data = CI_LB, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
matplot_y = data.frame(pointestroll,CI_LB,CI_UB)
matplot_x = c(1:6)
pdf(file = "AR_Forecast.pdf")
matplot(matplot_x,matplot_y, type = c('o','l','l'), col = c(1,4,4), pch = 15, lwd = 2, lty = c(1,2,2),main= 'One-hour ahead rolling AR(1) forecast', ylim = c(2,12), ylab = "Wind speed (m/s)", xlab ="Forecast time (hours)",cex.axis = 1.33, cex.lab = 1.33, cex.main =1.5)
legend("topright",legend = c("Point forecast","95% C. I."), lty = c(1,2), col = c(1,4))
dev.off()

## Kalman Filter
p = 1
var.eps.ref = 1
kappa = 1
nu = 1
a = 1
P = 1
Phi = 1
var.om = 0
h = wsdata$NormalizedWindspeed[427]
Vpred = NULL
CI_L = NULL
CI_U = NULL
Vobs = NULL
for (i in 1:6){
  var.eps = nu*var.eps.ref
  a = Phi*a
  P = Phi*P*Phi + var.om
  Vpred[i] = h*a
  var.V = h*P*h + var.eps
  CI_L[i] = Vpred[i] - 1.96*(var.V)
  CI_U[i] = Vpred[i] + 1.96*(var.V)
  Vobs[i] = wsdata$NormalizedWindspeed[427+i]
  nu = (nu/(kappa+1))*(kappa + (((Vobs[i] - Vpred[i])^2)/var.V))
  kappa = kappa + 1
  var.om = (((Vobs[i] - Vpred[i])^2) - (h*Phi*P*Phi*h) - var.eps)/(h*h)
  if (var.om<0){var.om = 0}
  K = (1/var.V)*P*h
  a = a + K*(Vobs[i] - Vpred[i] )
  P = (1 - K*h)*P
  h = wsdata$NormalizedWindspeed[427+i]
}
Vpred = unnormalize(data = Vpred, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
CI_U = unnormalize(data = CI_U, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
CI_L = unnormalize(data = CI_L, time = wsdata$Time[(nrow(wsdata)-5):nrow(wsdata)], refdata = ws)
matplot_y = data.frame(Vpred,CI_L,CI_U)
matplot_x = c(1:6)
pdf(file = 'KF_Forecast.pdf')
matplot(matplot_x,matplot_y, type = c('o','l','l'),pch = 19, col = c(1,4,4), lwd = 2, lty = c(1,2,2),main= 'Kalman filter forecast', ylab = "Wind speed (m/s)", xlab ="Forecast time (hours)",cex.axis = 1.33, cex.lab = 1.33, cex.main =1.5)
legend("topright",legend = c("Point forecast","95% C. I."), lty = c(1,2), col = c(1,4))
dev.off()
