### Section 2.7 PER
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
pred = NULL
testdata = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
for (j in 1:length(Month)){
  wsdata = ws[grep(Month[j],ws$Time),]
  traindata = wsdata[(1:(nrow(wsdata)-6)),2]
  testmonth = wsdata[((nrow(wsdata)-5):nrow(wsdata)),2]
  testdata = rbind(testdata,testmonth)
  predmonth = NULL
  for (i in 1:6){
    predmonth = cbind(predmonth,traindata[length(traindata)])
  }
  pred = rbind(pred, predmonth)
}
for (i in 1:6){
  RMSE[i] = sqrt((1/nrow(testdata))*sum((testdata[,i]-pred[,i])^2))
  MAE[i] = (1/nrow(testdata))*sum(abs(testdata[,i]-pred[,i]))
  MAPE[i] = (100/nrow(testdata))*sum(abs((testdata[,i]-pred[,i])/testdata[,i]))
}
sink(file = "PER.txt")
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()

### WEB
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
pred = NULL
testdata = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
mean.weibull = function (scale, shape){
  scale * (gamma(1+(1/shape)))
}
for (j in 1:length(Month)){
  wsdata = ws[grep(Month[j],ws$Time),]
  traindata = wsdata[(1:(nrow(wsdata)-6)),2]
  testmonth = wsdata[((nrow(wsdata)-5):nrow(wsdata)),2]
  testdata = rbind(testdata,testmonth)
  predmonth = NULL
  library(MASS)
  weibull.fit =  fitdistr(traindata, "weibull")
  shape = weibull.fit$estimate[1]
  scale = weibull.fit$estimate[2]
  for (i in 1:6){
    predmonth = cbind(predmonth,mean.weibull(scale,shape))
  }
  pred = rbind(pred, predmonth)
}
for (i in 1:6){
  RMSE[i] = sqrt((1/nrow(testdata))*sum((testdata[,i]-pred[,i])^2))
  MAE[i] = (1/nrow(testdata))*sum(abs(testdata[,i]-pred[,i]))
  MAPE[i] = (100/nrow(testdata))*sum(abs((testdata[,i]-pred[,i])/testdata[,i]))
}
sink(file = "WEB.txt")
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()

### ARMA
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
wsdata.train = NULL
wsdata.test = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
bestmodel = NULL
pred = NULL
testdata = NULL
for (k in 1:length(Month)){
  wsdata.month = ws[grep(Month[k],ws$Time),]
  train.month= wsdata.month[(1:(nrow(wsdata.month)-6)),]
  test.month = wsdata.month[((nrow(wsdata.month)-5):nrow(wsdata.month)),]
  wsdata.train = rbind(wsdata.train,train.month)
  wsdata.test = rbind(wsdata.test,test.month)
}
wsdata.train.norm = normalize(wsdata.train) 
wsdata.test = normalize(wsdata.test)
p = c(1:4)
d = 0
q = c(1:3)
bestbic = Inf
for (i in p){
  for (j in q){
    arimafit<-arima(wsdata.train.norm[,5], order = c(p[i],d,q[j]))
    bic = BIC(arimafit)
    if (bic < bestbic){
      bestbic = bic
      bestmodel[1] = i
      bestmodel[2] = j
    }
  }
}
best.fit = arima(wsdata.train.norm[,5], order = c(bestmodel[1],d,bestmodel[2]))
bestp = bestmodel[1]
for (k in 1:length(Month)){
  test_y.month =wsdata.test[grep(Month[k],wsdata.test$Date),1:3]
  wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
  traindata = wsdata.month[,5]
  test_x.month = traindata[(length(traindata)-bestp+1):length(traindata)]
  forecast = forecast(test_x.month,6, model = best.fit)
  pointest = forecast$mean
  pointest = unnormalize(data = pointest, time = test_y.month$Time, refdata = wsdata.train)
  predmonth = NULL
  testdata = rbind(testdata,test_y.month[,3])
  for (i in 1:6){
    predmonth = cbind(predmonth,pointest[i])
  }
  pred = rbind(pred, predmonth)
}

for (i in 1:6){
  RMSE[i] = sqrt((1/nrow(testdata))*sum((testdata[,i]-pred[,i])^2))
  MAE[i] = (1/nrow(testdata))*sum(abs(testdata[,i]-pred[,i]))
  MAPE[i] = (100/nrow(testdata))*sum(abs((testdata[,i]-pred[,i])/testdata[,i]))
}
sink(file = "ARMA.txt")
print("Best Model:")
bestmodel
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()


### SVM
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
library(e1071)
wsdata.train = NULL
wsdata.test = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
bestmodel = matrix(nrow = 6, ncol = 3)
for (k in 1:length(Month)){
  wsdata.month = ws[grep(Month[k],ws$Time),]
  train.month= wsdata.month[(1:(nrow(wsdata.month)-6)),]
  test.month = wsdata.month[((nrow(wsdata.month)-5):nrow(wsdata.month)),]
  wsdata.train = rbind(wsdata.train,train.month)
  wsdata.test = rbind(wsdata.test,test.month)
}
wsdata.train.norm = normalize(wsdata.train) 
wsdata.test = normalize(wsdata.test)
for (h in 1:6){
  best.cv.err = Inf
  p = 0
  while (p < 4) {
    trainset = NULL
    p = p+1
    for (k in 1:length(Month)){
      wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
      traindata = wsdata.month$NormalizedWindspeed
      x = matrix(nrow = (length(traindata)-p-h+1), ncol = p)
      y = NULL
      for (i in 1:nrow(x)){
        x[i,] = traindata[i:(p-1+i)]
        y[i] =  traindata[p+i+h-1]
      }
      trainset.month = data.frame(x,y)
      trainset = rbind(trainset,trainset.month)
      
    }
    trainset_x = trainset[,1:p]
    trainset_y = trainset[,p+1]
    set.seed(1)
    outcome = tune(svm, train.x = trainset_x, train.y = trainset_y, ranges = list(epsilon = seq(0,1,0.1), cost = c(0.01,0.1,1,10,100)), kernel = "radial", gamma = 1/p,tunecontrol = tune.control(cross = 5))
    cv.err = outcome$best.performance
    if (cv.err<best.cv.err){
      best.cv.err = cv.err
      bestmodel[h,1] = p
      bestmodel[h,2] = outcome$best.model$cost
      bestmodel[h,3] = outcome$best.model$epsilon
      bestmod = outcome$best.model 
    }
  }
  bestp = bestmodel[h,1]
  test_x = NULL
  test_y = NULL
  for (k in 1:length(Month)){
    test_y = rbind(test_y,wsdata.test[(6*(k-1)+h),2:3])
    wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
    traindata = wsdata.month$NormalizedWindspeed
    test_x.month = traindata[(length(traindata)-bestp+1):length(traindata)]
    test_x = rbind(test_x,test_x.month)
  }
  ypred = predict(bestmod,test_x)
  ypred = unnormalize(data=ypred,time=test_y$Time,refdata = wsdata.train)
  RMSE[h] = sqrt((1/length(ypred))*sum((test_y$WindSpeed-ypred)^2))
  MAE[h] = (1/length(ypred))*sum(abs(test_y$WindSpeed-ypred))
  MAPE[h] = (100/length(ypred))*sum(abs((test_y$WindSpeed-ypred)/test_y$WindSpeed))
}

sink(file = "SVM.txt")
print("Best Model:")
bestmodel
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()


### ANN
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
library(neuralnet)
wsdata.train = NULL
wsdata.test = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
bestmodel = matrix(nrow = 6, ncol = 2)
hidden_nodes = c(5,10,20,30,40)
for (k in 1:length(Month)){
  wsdata.month = ws[grep(Month[k],ws$Time),]
  train.month= wsdata.month[(1:(nrow(wsdata.month)-6)),]
  test.month = wsdata.month[((nrow(wsdata.month)-5):nrow(wsdata.month)),]
  wsdata.train = rbind(wsdata.train,train.month)
  wsdata.test = rbind(wsdata.test,test.month)
}
wsdata.train.norm = normalize(wsdata.train) 
wsdata.test = normalize(wsdata.test)
for (h in 1:6){
  best.cv.err = Inf
  p = 0
  while (p < 4) {
    trainset = NULL
    p = p+1
    for (k in 1:length(Month)){
      wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
      traindata = wsdata.month$NormalizedWindspeed
      x = matrix(nrow = (length(traindata)-p-h+1), ncol = p)
      y = NULL
      for (i in 1:nrow(x)){
        x[i,] = traindata[i:(p-1+i)]
        y[i] =  traindata[p+i+h-1]
      }
      trainset.month = data.frame(x,y)
      trainset = rbind(trainset,trainset.month)
    }
    trainset_x = trainset[,1:p]
    trainset_y = trainset[,p+1]
    sample = sample(nrow(trainset))
    folds = cut(seq(1:length(sample)),breaks = 5, labels = F)
    i = 1
    for (i in 1:length(hidden_nodes)){
      err = NULL
      j = 1
      for(j in 1:5){
        testindex = sample(which(folds == j,arr.ind = T))
        cv.trainset = trainset[-testindex,]
        cv.testset_y = trainset_y[testindex]
        if (p==1){
          cv.testset_x = trainset_x[testindex]
        }else {
          cv.testset_x = trainset_x[testindex,]
        }
        
        if (p==1){
          nn = neuralnet(y~x, data = cv.trainset, hidden = hidden_nodes[i], linear.output = T, stepmax = 1e9)
          test.nn = compute(nn, cv.testset_x)
          pred = test.nn$net.result
          err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
        } 
        else if (p==2){
          nn = neuralnet(y~X1+X2, data = cv.trainset, hidden = hidden_nodes[i], linear.output = T, stepmax = 1e9)
          test.nn = compute(nn, cv.testset_x)
          pred = test.nn$net.result
          err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
        } 
        else if (p==3){
          nn = neuralnet(y~X1+X2+X3, data = cv.trainset, hidden = hidden_nodes[i], linear.output = T, stepmax = 1e9)
          test.nn = compute(nn, cv.testset_x)
          pred = test.nn$net.result
          err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
        } 
        else if (p==4){
          nn = neuralnet(y~X1+X2+X3+X4, data = cv.trainset, hidden = hidden_nodes[i], linear.output = T, stepmax = 1e9)
          test.nn = compute(nn, cv.testset_x)
          pred = test.nn$net.result
          err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
        }
      }
      cv.err = mean(err)
      if (cv.err<best.cv.err){
        best.cv.err = cv.err
        bestmodel[h,1] = p
        bestmodel[h,2] = hidden_nodes[i]
      }
    }
  }
  bestp = bestmodel[h,1]
  best.hidden_nodes = bestmodel[h,2]
  trainset = NULL
  for (k in 1:length(Month)){
    wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
    traindata = wsdata.month$NormalizedWindspeed
    x = matrix(nrow = (length(traindata)-bestp-h+1), ncol = bestp)
    y = NULL
    for (i in 1:nrow(x)){
      x[i,] = traindata[i:(bestp-1+i)]
      y[i] =  traindata[bestp+i+h-1]
    }
    trainset.month = data.frame(x,y)
    trainset = rbind(trainset,trainset.month)
  }
  if (bestp==1){
    fullnn = neuralnet(y~x, data = trainset, hidden = best.hidden_nodes, linear.output = T, stepmax = 1e9)
  } else if (bestp==2){
    fullnn = neuralnet(y~X1+X2, data = trainset, hidden = best.hidden_nodes, linear.output = T, stepmax = 1e9)
  } else if (bestp==3){
    fullnn = neuralnet(y~X1+X2+X3, data = trainset, hidden = best.hidden_nodes, linear.output = T, stepmax = 1e9)
  }else if (bestp==4){
    fullnn = neuralnet(y~X1+X2+X3+X4, data = trainset, hidden = best.hidden_nodes, linear.output = T, stepmax = 1e9)
  }
  test_x = NULL
  test_y = NULL
  for (k in 1:length(Month)){
    test_y = rbind(test_y,wsdata.test[(6*(k-1)+h),2:3])
    wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
    traindata = wsdata.month$NormalizedWindspeed
    test_x.month = traindata[(length(traindata)-bestp+1):length(traindata)]
    test_x = rbind(test_x,test_x.month)
  }
  ypred = compute(fullnn,test_x)
  ypred = unnormalize(data=ypred,time=test_y$Time,refdata = wsdata.train)
  RMSE[h] = sqrt((1/length(ypred))*sum((test_y$WindSpeed-ypred)^2))
  MAE[h] = (1/length(ypred))*sum(abs(test_y$WindSpeed-ypred))
  MAPE[h] = (100/length(ypred))*sum(abs((test_y$WindSpeed-ypred)/test_y$WindSpeed))
}
sink(file = "ANN.txt")
print("Best Model:")
bestmodel
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()    

### Page 48 ANN - Modified (Cross Validation done only using April month data)
source('bookfunctions.r')
ws = read.csv(file = "Wind Time Series Data(Hourly).csv", header = T)
Month = c("-01-","-02-","-03-","-04-","-05-","-06-","-07-","-08-","-09-","-10-","-11-","-12-")
library(neuralnet)
wsdata.train = NULL
wsdata.test = NULL
RMSE = NULL
MAE = NULL
MAPE = NULL
bestmodel = matrix(nrow = 6, ncol = 2)
wsdata.norm = normalize(ws) 
hidden_nodes = c(5,10,15,30)
wsdata.month = wsdata.norm[grep(Month[4],wsdata.norm$Date),]
train.month= wsdata.month[(1:(nrow(wsdata.month)-6)),]
test.month = wsdata.month[((nrow(wsdata.month)-5):nrow(wsdata.month)),]
for (h in 1:6){
  best.cv.err = Inf
  p = 1
  traindata = train.month$NormalizedWindspeed
  x = matrix(nrow = (length(traindata)-p-h+1), ncol = p)
  y = NULL
  for (i in 1:nrow(x)){
    x[i,] = traindata[i:(p-1+i)]
    y[i] =  traindata[p+i+h-1]
  }
  trainset.month = data.frame(x,y)
  trainset_x = trainset.month[,1:p]
  trainset_y = trainset.month[,p+1]
  sample = sample(nrow(trainset.month))
  folds = cut(seq(1:length(sample)),breaks = 10, labels = F)
  for (i in 1:length(hidden_nodes)){
    err = NULL
    for(j in 1:10){
      testindex = sample(which(folds == j,arr.ind = T))
      cv.trainset = trainset.month[-testindex,]
      cv.testset_y = trainset_y[testindex]
      cv.testset_x = trainset_x[testindex]
      nn = neuralnet(y~x, data = cv.trainset, hidden = hidden_nodes[i], linear.output = T)
      test.nn = compute(nn, cv.testset_x)
      pred = test.nn$net.result
      err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
    }
    cv.err = mean(err)
    if (cv.err<best.cv.err){
      best.cv.err = cv.err
      bestmodel[h,2] = hidden_nodes[i]
    }
  }
  p = 0
  best.cv.err = Inf
  while (p < 4) {
    p = p+1
    traindata = train.month$NormalizedWindspeed
    x = matrix(nrow = (length(traindata)-p-h+1), ncol = p)
    y = NULL
    for (i in 1:nrow(x)){
      x[i,] = traindata[i:(p-1+i)]
      y[i] =  traindata[p+i+h-1]
    }
    trainset.month = data.frame(x,y)
    trainset_x = trainset.month[,1:p]
    trainset_y = trainset.month[,p+1]
    sample = sample(nrow(trainset.month))
    folds = cut(seq(1:length(sample)),breaks = 10, labels = F)
    err = NULL
    for(j in 1:10){
      testindex = sample(which(folds == j,arr.ind = T))
      cv.trainset = trainset.month[-testindex,]
      cv.testset_y = trainset_y[testindex]
      if (p==1){
        cv.testset_x = trainset_x[testindex]
      }else {
        cv.testset_x = trainset_x[testindex,]
      }
      
      if (p==1){
        nn = neuralnet(y~x, data = cv.trainset, hidden = bestmodel[h,2], linear.output = T)
        test.nn = compute(nn, cv.testset_x)
        pred = test.nn$net.result
        err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
      } 
      else if (p==2){
        nn = neuralnet(y~X1+X2, data = cv.trainset, hidden = bestmodel[h,2], linear.output = T)
        test.nn = compute(nn, cv.testset_x)
        pred = test.nn$net.result
        err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
      } 
      else if (p==3){
        nn = neuralnet(y~X1+X2+X3, data = cv.trainset, hidden = bestmodel[h,2], linear.output = T)
        test.nn = compute(nn, cv.testset_x)
        pred = test.nn$net.result
        err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
      } 
      else if (p==4){
        nn = neuralnet(y~X1+X2+X3+X4, data = cv.trainset, hidden = bestmodel[h,2], linear.output = T)
        test.nn = compute(nn, cv.testset_x)
        pred = test.nn$net.result
        err[j] = sqrt((1/length(pred))*sum((cv.testset_y-pred)^2))
      }
    }
    cv.err = mean(err)
    if (cv.err<best.cv.err){
      best.cv.err = cv.err
      bestmodel[h,1] = p
    }
  }
}
for (h in 1:6){
  bestp = bestmodel[h,1]
  best.hidden_nodes = bestmodel[h,2]
  wsdata.train = NULL
  wsdata.test = NULL
  for (k in 1:length(Month)){
    wsdata.month = ws[grep(Month[k],ws$Time),]
    train.month= wsdata.month[(1:(nrow(wsdata.month)-6)),]
    test.month = wsdata.month[((nrow(wsdata.month)-5):nrow(wsdata.month)),]
    wsdata.train = rbind(wsdata.train,train.month)
    wsdata.test = rbind(wsdata.test,test.month)
  }
  wsdata.train.norm = normalize(wsdata.train) 
  wsdata.test = normalize(wsdata.test)
  trainset = NULL
  for (k in 1:length(Month)){
    wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
    traindata = wsdata.month$NormalizedWindspeed
    x = matrix(nrow = (length(traindata)-bestp-h+1), ncol = bestp)
    y = NULL
    for (i in 1:nrow(x)){
      x[i,] = traindata[i:(bestp-1+i)]
      y[i] =  traindata[bestp+i+h-1]
    }
    trainset.month = data.frame(x,y)
    trainset = rbind(trainset,trainset.month)
  }
  if (bestp==1){
    fullnn = neuralnet(y~x, data = trainset, hidden = best.hidden_nodes, linear.output = T)
  } else if (bestp==2){
    fullnn = neuralnet(y~X1+X2, data = trainset, hidden = best.hidden_nodes, linear.output = T)
  } else if (bestp==3){
    fullnn = neuralnet(y~X1+X2+X3, data = trainset, hidden = best.hidden_nodes, linear.output = T)
  }else if (bestp==4){
    fullnn = neuralnet(y~X1+X2+X3+X4, data = trainset, hidden = best.hidden_nodes, linear.output = T)
  }
  test_x = NULL
  test_y = NULL
  for (k in 1:length(Month)){
    test_y = rbind(test_y,wsdata.test[(6*(k-1)+h),2:3])
    wsdata.month = wsdata.train.norm[grep(Month[k],wsdata.train.norm$Date),]
    traindata = wsdata.month$NormalizedWindspeed
    test_x.month = traindata[(length(traindata)-bestp+1):length(traindata)]
    test_x = rbind(test_x,test_x.month)
  }
  ypred = compute(fullnn,test_x)
  ypred = ypred$net.result
  ypred = unnormalize(data=ypred,time=test_y$Time,refdata = wsdata.train)
  RMSE[h] = sqrt((1/length(ypred))*sum((test_y$WindSpeed-ypred)^2))
  MAE[h] = (1/length(ypred))*sum(abs(test_y$WindSpeed-ypred))
  MAPE[h] = (100/length(ypred))*sum(abs((test_y$WindSpeed-ypred)/test_y$WindSpeed))
}
sink(file = "ANN_Modified.txt")
print("Best Model:")
bestmodel
print("RMSE:") 
RMSE
print("MAE:")
MAE
print("MAPE:")
MAPE
sink()    
